@@ -1,4 +1,4 @@
-# moved from Rcmdr 19 November 2004
+# moved from Rcmdr 5 February 2006
 
 # the following function adapted from Fox, An R and S-PLUS Companion to Applied Regression
 influence.plot <- function(model, ...){
@@ -6,7 +6,7 @@ influence.plot <- function(model, ...){
     }
 
 influence.plot.lm <- function(model, scale=10, col=c(1,2),
-    labels=names(rstud), ...){
+    labels=names(rstud), identify.cex=par("cex"), identify.col=par("col"), ...){
     hatval <- hatvalues(model)
     rstud <- rstudent(model)
     cook <- sqrt(cookd(model))
@@ -20,5 +20,6 @@ influence.plot.lm <- function(model, scale=10, col=c(1,2),
     abline(h=c(-2, 0, 2), lty=2)
     points(hatval, rstud, cex=scale*cook, 
             col=ifelse(cook > cutoff, col[2], col[1]))
-    if (labels[1] != FALSE) identify(hatval, rstud, labels)
+    if (labels[1] != FALSE) identify(hatval, rstud, labels, col=identify.col,
+        cex=identify.cex)
     }
